package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import net.knarcraft.ffmpegconverter.utility.FileUtil;
import net.knarcraft.ffmpegconverter.utility.OutputUtil;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * A converter mainly designed for converting anime to web-playable mp4
 */
public class AnimeConverter extends AbstractConverter {
    private final String[] audioLang;
    private final String[] subtitleLang;
    private final boolean toStereo;
    private final boolean preventSignsAndSongs;

    /**
     * Instantiates a new anime converter
     *
     * @param ffprobePath          <p>Path/command to ffprobe.</p>
     * @param ffmpegPath           <p>Path/command to ffmpeg.</p>
     * @param audioLang            <p>List of wanted audio languages in descending order.</p>
     * @param subtitleLang         <p>List of wanted subtitle languages in descending order.</p>
     * @param toStereo             <p>Convert video with several audio channels to stereo.</p>
     * @param preventSignsAndSongs <p>Prevent subtitles only converting signs and songs (not speech).</p>
     */
    public AnimeConverter(String ffprobePath, String ffmpegPath, String[] audioLang, String[] subtitleLang,
                          boolean toStereo, boolean preventSignsAndSongs) {
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.audioLang = audioLang;
        this.subtitleLang = subtitleLang;
        this.toStereo = toStereo;
        this.preventSignsAndSongs = preventSignsAndSongs;
    }

    @Override
    public void convert(File file) throws IOException {
        processFile(file.getParentFile(), file);
    }

    /**
     * Reads streams from a file, and converts it to an mp4
     *
     * @param folder <p>The folder of the file to process.</p>
     * @param file   <p>The file to process.</p>
     * @throws IOException <p>If the BufferedReader fails.</p>
     */
    private void processFile(File folder, File file) throws IOException {
        List<StreamObject> streams = FFMpegHelper.probeFile(ffprobePath, file);
        if (streams.isEmpty()) {
            throw new IllegalArgumentException("The file has no valid streams. Please make sure the file exists and" +
                    " is not corrupt.");
        }
        String newPath = FileUtil.getNonCollidingFilename(folder.getAbsolutePath() + File.separator +
                FileUtil.stripExtension(file) + ".mp4", "mp4");
        OutputUtil.println();
        OutputUtil.println("Preparing to start process...");
        OutputUtil.println("Converting " + file);
        String[] command = builderCommand(ffmpegPath, file.getName(), streams, newPath, file);
        ProcessBuilder processBuilder = new ProcessBuilder(command);
        FFMpegHelper.convertProcess(processBuilder, folder);
    }

    /**
     * Generates a command for a ProcessBuilder.
     *
     * @param executable <p>The executable file for ffmpeg.</p>
     * @param fileName   <p>The input file.</p>
     * @param streams    <p>A list of ffprobe streams.</p>
     * @param outFile    <p>The output file.</p>
     * @return <p>A list of commands</p>
     */
    private String[] builderCommand(String executable, String fileName, List<StreamObject> streams, String outFile,
                                    File file) {
        List<String> command = FFMpegHelper.getFFMpegWebVideoCommand(executable, fileName);

        if (this.DEBUG) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        List<AudioStream> audioStreams = filterAudioStreams(filterStreamsByType(streams, "audio"), audioLang);
        List<VideoStream> videoStreams = filterStreamsByType(streams, "video");
        List<SubtitleStream> subtitleStreams = filterSubtitleStreams(filterStreamsByType(streams,
                "subtitle"), subtitleLang, preventSignsAndSongs);

        VideoStream videoStream = null;
        AudioStream audioStream = null;
        SubtitleStream subtitleStream = null;
        if (videoStreams.size() > 0) {
            videoStream = videoStreams.get(0);
        }
        if (videoStream == null) {
            throw new IllegalArgumentException("The file does not have any valid video streams.");
        }
        if (audioStreams.size() > 0) {
            audioStream = audioStreams.get(0);
        }
        if (subtitleStreams.size() > 0) {
            subtitleStream = subtitleStreams.get(0);
        }

        addAudioStreams(command, audioStream, toStereo);
        addSubtitles(command, subtitleStream, videoStream, file);

        command.add(outFile);
        return command.toArray(new String[0]);
    }

    @Override
    public String[] getValidFormats() {
        return videoFormats;
    }
}
